@@ -11,7 +11,7 @@
  * no representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tSippBezier.c,v 3.0 1993/01/20 06:43:29 markd Rel $
+ * $Id: tSippBezier.c,v 5.0 1994/09/05 01:23:09 markd Rel $
  *=============================================================================
  */
 
@@ -478,64 +478,6 @@ SippBezierPatch (clientData, interp, argc, argv)
 
 }
 
-/*=============================================================================
- * SippBezierFile --
- *   Implements the command:
- *      SippBezierFile fileid resolution shaderhandle [texture]
- * Note:
- *   This procedure has standard Tcl command calling sematics.  ClientData
- * contains a pointer to the Tcl SIPP global structure.
- *-----------------------------------------------------------------------------
- */
-int
-SippBezierFile (clientData, interp, argc, argv)
-    char       *clientData;
-    Tcl_Interp *interp;
-    int         argc;
-    char      **argv;
-{
-    tSippGlob_pt   tSippGlobPtr = (tSippGlob_pt) clientData;
-    Shader        *shaderPtr;
-    void          *surfDescPtr;
-    int            resolution, texture = NATURAL;
-    OpenFile      *filePtr;
-
-    if (tSippGlobPtr->rendering)
-        return TSippNotWhileRendering (interp);
-
-    if ((argc < 4) || (argc > 5)) {
-        Tcl_AppendResult (interp, "wrong # args: ", argv [0],
-                          " fileid resolution shaderhandle [texture]",
-                          (char *) NULL);
-        return TCL_ERROR;
-    }
-
-    if (TclGetOpenFile (interp, argv [1], &filePtr) != TCL_OK)
-        return TCL_ERROR;
-
-    if (!TSippConvertPosUnsigned (tSippGlobPtr, argv [2], &resolution))
-        return TCL_ERROR;
-
-    shaderPtr = TSippShaderHandleToPtr (tSippGlobPtr, argv [3],
-                                        &surfDescPtr);
-    if (shaderPtr == NULL)
-        return TCL_ERROR;
-
-    if (argc == 5) {
-        if (!TSippParseTextureMapping (tSippGlobPtr, argv [4], &texture,
-                                       invalidTextureMappings))
-            return TCL_ERROR;
-    }
-
-    TSippBindObjectToHandle (tSippGlobPtr, 
-                             sipp_bezier_file (filePtr->f, resolution,
-                                               surfDescPtr, shaderPtr,
-                                               texture));
-
-    return TCL_OK;
-
-}
-
 /*=============================================================================
  * TSippBezierInit --
  *   Initialized the polygon and surface commands, including creating the 
@@ -552,7 +494,6 @@ TSippBezierInit (tSippGlobPtr)
     static tSippTclCmdTbl_t cmdTable [] = {
         {"SippBezierCurve",      (Tcl_CmdProc *) SippBezierCurve},
         {"SippBezierPatch",      (Tcl_CmdProc *) SippBezierPatch},
-        {"SippBezierFile",       (Tcl_CmdProc *) SippBezierFile},
         {NULL,                   NULL}
     };
 
