@@ -11,7 +11,7 @@
  * no representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tSippGeom.c,v 3.0 1993/01/20 06:44:03 markd Rel $
+ * $Id: tSippGeom.c,v 5.0 1994/09/05 01:23:20 markd Rel $
  *============================================================================
  */
 
@@ -51,24 +51,26 @@ ReturnVector (tSippGlobPtr, vectorPtr)
     tSippGlob_pt   tSippGlobPtr;
     Vector        *vectorPtr;
 {
-    char  *bufPtr;
+    char xBuf [TCL_DOUBLE_SPACE];
+    char yBuf [TCL_DOUBLE_SPACE];
+    char zBuf [TCL_DOUBLE_SPACE];
 
-    /*
-     * Allocate a dynamic buffer only if there is a chance of the result not
-     * fitting into the standard Tcl result.
-     */
-    if (TCL_RESULT_SIZE < (TSIPP_DOUBLE_STR_SIZE * 3)) {
-        Tcl_SetResult (tSippGlobPtr->interp,
-                       smalloc (TSIPP_DOUBLE_STR_SIZE * 3),
-                       TCL_DYNAMIC);
-    }
-
-    bufPtr = TSippFormatDouble (vectorPtr->x, tSippGlobPtr->interp->result);
-    *bufPtr = ' ';
-    bufPtr = TSippFormatDouble (vectorPtr->y, bufPtr + 1);
-    *bufPtr = ' ';
-    TSippFormatDouble (vectorPtr->z, bufPtr + 1);
+    Tcl_PrintDouble (tSippGlobPtr->interp,
+                     vectorPtr->x,
+                     xBuf);
+                     
+    Tcl_PrintDouble (tSippGlobPtr->interp,
+                     vectorPtr->y,
+                     yBuf);
+                     
+    Tcl_PrintDouble (tSippGlobPtr->interp,
+                     vectorPtr->z,
+                     zBuf);
 
+    Tcl_AppendResult (tSippGlobPtr->interp,
+                      xBuf, " ",
+                      yBuf, " ",
+                      zBuf, (char *) NULL);
 }
 
 /*=============================================================================
@@ -174,7 +176,9 @@ SippVecDot (clientData, interp, argc, argv)
     if (!TSippConvertVertex (tSippGlobPtr, argv [2], &vector1))
         return TCL_ERROR;
 
-    TSippFormatDouble (VecDot (vector0, vector1), interp->result);
+    Tcl_PrintDouble (interp, 
+                     VecDot (vector0, vector1),
+                     interp->result);
     return TCL_OK;
 
 }
@@ -207,7 +211,9 @@ SippVecLen (clientData, interp, argc, argv)
     if (!TSippConvertVertex (tSippGlobPtr, argv [1], &vector))
         return TCL_ERROR;
 
-    TSippFormatDouble (VecLen (vector), interp->result);
+    Tcl_PrintDouble (interp,
+                     VecLen (vector),
+                     interp->result);
     return TCL_OK;
 
 }
@@ -284,6 +290,44 @@ SippVecSub (clientData, interp, argc, argv)
 
 }
 
+/*=============================================================================
+ * SippVecMult --
+ *   Implements the command:
+ *     SippVecMult number vector
+ * Note:
+ *   This procedure has standard Tcl command calling sematics.  ClientData
+ * contains a pointer to the Tcl SIPP global structure.
+ *-----------------------------------------------------------------------------
+ */
+static int
+SippVecMult (clientData, interp, argc, argv)
+    char       *clientData;
+    Tcl_Interp *interp;
+    int         argc;
+    char      **argv;
+{
+    tSippGlob_pt  tSippGlobPtr = (tSippGlob_pt) clientData;
+    Vector        vector, result;
+    double        number;
+
+    if (argc != 3) {
+        Tcl_AppendResult (interp, "wrong # args: ", argv [0],
+                          " number vector", (char *) NULL);
+        return TCL_ERROR;
+    }                     
+
+    if (Tcl_GetDouble (interp, argv [1], &number) != TCL_OK)
+        return TCL_ERROR;
+
+    if (!TSippConvertVertex (tSippGlobPtr, argv [2], &vector))
+        return TCL_ERROR;
+
+    VecScalMul (result, number, vector);
+    ReturnVector (tSippGlobPtr, &result);
+    return TCL_OK;
+
+}
+
 /*=============================================================================
  * SippVecCross --
  *   Implements the command:
@@ -343,7 +387,9 @@ SippMatIdent (clientData, interp, argc, argv)
                           (char *) NULL);
         return TCL_ERROR;;
     }
-    Tcl_SetResult (tSippGlobPtr->interp, TSippFormatMatrix (&ident_matrix),
+    Tcl_SetResult (tSippGlobPtr->interp,
+                   TSippFormatMatrix (tSippGlobPtr,
+                                      &ident_matrix),
                    TCL_DYNAMIC);
     return TCL_OK;
     
@@ -381,7 +427,9 @@ SippMatTranslate (clientData, interp, argc, argv)
 
     mat_translate (&matrix, vector.x, vector.y, vector.z);
 
-    Tcl_SetResult (tSippGlobPtr->interp, TSippFormatMatrix (&matrix),
+    Tcl_SetResult (tSippGlobPtr->interp,
+                   TSippFormatMatrix (tSippGlobPtr,
+                                      &matrix),
                    TCL_DYNAMIC);
     return TCL_OK;
 
@@ -422,7 +470,9 @@ MatrixAxisRotate (tSippGlobPtr, argc, argv, rotateFunc)
         return FALSE;
  
     (*rotateFunc) (&matrix, angle);
-    Tcl_SetResult (tSippGlobPtr->interp, TSippFormatMatrix (&matrix),
+    Tcl_SetResult (tSippGlobPtr->interp,
+                   TSippFormatMatrix (tSippGlobPtr,
+                                      &matrix),
                    TCL_DYNAMIC);
     return TRUE;
 
@@ -535,7 +585,9 @@ SippMatRotate (clientData, interp, argc, argv)
         return TCL_ERROR;
 
     mat_rotate (&matrix, &point, &vector, angle);
-    Tcl_SetResult (tSippGlobPtr->interp, TSippFormatMatrix (&matrix),
+    Tcl_SetResult (tSippGlobPtr->interp,
+                   TSippFormatMatrix (tSippGlobPtr,
+                                      &matrix),
                    TCL_DYNAMIC);
     return TCL_OK;
 
@@ -585,7 +637,9 @@ SippMatScale (clientData, interp, argc, argv)
 
     mat_scale (&matrix, scale.x, scale.y, scale.z);
 
-    Tcl_SetResult (tSippGlobPtr->interp, TSippFormatMatrix (&matrix),
+    Tcl_SetResult (tSippGlobPtr->interp,
+                   TSippFormatMatrix (tSippGlobPtr,
+                                      &matrix),
                    TCL_DYNAMIC);
     return TCL_OK;
 
@@ -625,7 +679,9 @@ SippMatMirrorPlane (clientData, interp, argc, argv)
 
     mat_mirror_plane (&matrix, &point, &normal);
 
-    Tcl_SetResult (tSippGlobPtr->interp, TSippFormatMatrix (&matrix),
+    Tcl_SetResult (tSippGlobPtr->interp,
+                   TSippFormatMatrix (tSippGlobPtr,
+                                      &matrix),
                    TCL_DYNAMIC);
     return TCL_OK;
 
@@ -662,7 +718,10 @@ SippMatMult (clientData, interp, argc, argv)
 
     mat_mul (&resultMat, &matrix0, &matrix1);
     
-    Tcl_SetResult (interp, TSippFormatMatrix (&resultMat), TCL_DYNAMIC);
+    Tcl_SetResult (interp,
+                   TSippFormatMatrix (tSippGlobPtr,
+                                      &resultMat),
+                   TCL_DYNAMIC);
     return TCL_OK;
 
 }
@@ -723,6 +782,7 @@ TSippGeomInit (tSippGlobPtr)
         {"SippVecLen",          (Tcl_CmdProc *) SippVecLen},
         {"SippVecAdd",          (Tcl_CmdProc *) SippVecAdd},
         {"SippVecSub",          (Tcl_CmdProc *) SippVecSub},
+        {"SippVecMult",         (Tcl_CmdProc *) SippVecMult},
         {"SippVecCross",        (Tcl_CmdProc *) SippVecCross},
         {"SippMatIdent",        (Tcl_CmdProc *) SippMatIdent},
         {"SippMatTranslate",    (Tcl_CmdProc *) SippMatTranslate},
