@@ -11,7 +11,7 @@
  * no representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tSippPBM.c,v 3.0 1993/01/20 06:43:19 markd Rel $
+ * $Id: tSippPBM.c,v 5.0 1994/09/05 01:23:01 markd Rel $
  *=============================================================================
  */
 
@@ -31,8 +31,7 @@ typedef char fileHandle_t [10];
 typedef struct {
     fileHandle_t  handle;
     FILE         *filePtr;
-    bool          readable;
-    bool          writable;
+    int           perms;
     bool          raw;
 } pbmFile_t, *pbmFile_pt;
 
@@ -167,7 +166,8 @@ SippPBMOpen (clientData, interp, argc, argv)
     char           *openArgv [4];
     bool            raw = TRUE;
     bool            gotModeFlag = FALSE;
-    OpenFile       *filePtr;
+    FILE           *filePtr;
+    int             perms;
 
     if (tSippGlobPtr->rendering)
         return TSippNotWhileRendering (interp);
@@ -218,14 +218,16 @@ SippPBMOpen (clientData, interp, argc, argv)
     strcpy (fileHandle, interp->result);
     Tcl_ResetResult (interp);
 
-    if (TclGetOpenFile (interp, fileHandle, &filePtr) != TCL_OK)
-         return TCL_ERROR;  /* Should never happen */
+    if (Tcl_GetOpenFile (interp, fileHandle, FALSE, FALSE, &filePtr) != TCL_OK)
+        return TCL_ERROR;  /* Should never happen */
+
+    perms = Tcl_FilePermissions (filePtr);
 
     /*
      * We don't currently support reading of PBMPlus files because the
      * PBMPlus libraries exit on error rather than returning status!!!
      */
-    if (filePtr->readable) {
+    if (perms & TCL_FILE_READABLE) {
         ClosePBMFile (tSippGlobPtr, argv [0], pbmFilePtr);
         Tcl_ResetResult (interp);
         Tcl_AppendResult (interp, "read access to PBMPlus files is not ",
@@ -239,10 +241,9 @@ SippPBMOpen (clientData, interp, argc, argv)
     pbmFilePtr = (pbmFile_pt) Tcl_HandleAlloc (tSippGlobPtr->pbmTblPtr,
                                                interp->result);
     strcpy (pbmFilePtr->handle, fileHandle);
-    pbmFilePtr->readable  = filePtr->readable;
-    pbmFilePtr->writable  = filePtr->writable;
-    pbmFilePtr->filePtr   = filePtr->f;
-    pbmFilePtr->raw       = raw;
+    pbmFilePtr->perms = perms;
+    pbmFilePtr->filePtr = filePtr;
+    pbmFilePtr->raw = raw;
 
     return TCL_OK;
 
@@ -366,7 +367,7 @@ PBMOutputStart (tSippGlobPtr, outputParmsPtr, handle, comments)
     if (pbmFilePtr == NULL)
         return NULL;
 
-    if (!pbmFilePtr->writable) {
+    if ((pbmFilePtr->perms & TCL_FILE_WRITABLE) == 0) {
         TSippNotWritable (tSippGlobPtr->interp, handle);
         return NULL;
     }
