@@ -12,7 +12,7 @@
  * no representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tSippPixMap.c,v 3.0 1993/01/20 06:44:05 markd Rel $
+ * $Id: tSippPixMap.c,v 5.0 1994/09/05 01:23:41 markd Rel $
  *=============================================================================
  */
 
@@ -785,10 +785,10 @@ PixMapCopyImage (tSippGlobPtr, srcHandle, destHandle, destClassPtr, clear)
     /*
      * Initialize other output parms fields.
      */
-    outputParms.argv     = NULL;   
-    outputParms.update   = 0;
-    outputParms.maxDepth = 0;
-    outputParms.clear    = clear;
+    outputParms.tSippGlobPtr = tSippGlobPtr;
+    outputParms.argv = NULL;   
+    outputParms.update = 0;
+    outputParms.clear = clear;
 
     outputParms.bitMapOutput  = srcPixMapPtr->bitMapPtr != NULL;
     outputParms.scanDirection = destClassPtr->scanDirection;
