@@ -11,12 +11,18 @@
  * no representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tSippPoly.c,v 3.0 1993/01/20 06:43:40 markd Rel $
+ * $Id: tSippPoly.c,v 5.0 1994/09/05 01:23:23 markd Rel $
  *=============================================================================
  */
 
 #include "tSippInt.h"
-#include "primitives.h"
+
+/*
+ * Flags from parsed options to SippPolygonPush
+ */
+#define POLYGON_NORM     0x01   /* Normal supplied               */
+#define POLYGON_TEX      0x02   /* Textures supplied             */
+#define POLYGON_CLOCK    0x04   /* Clockwise push of coordinates */
 
 /*
  * Internal prototypes.
@@ -32,15 +38,16 @@ static void
 SurfaceHandleCleanup _ANSI_ARGS_((tSippGlob_pt   tSippGlobPtr));
 
 static bool
-PushVertexList _ANSI_ARGS_((tSippGlob_pt    tSippGlobPtr,
-                            bool            textures,
-                            bool            clockwise,
-                            char           *vertexList));
+ConvertVertexData _ANSI_ARGS_((tSippGlob_pt  tSippGlobPtr,
+                               char         *listStr,
+                               int           numTriples,
+                               Vector        vertexData []));
 
 static bool
-PushVertexArgList _ANSI_ARGS_((tSippGlob_pt    tSippGlobPtr,
-                               int             argc,    
-                               char          **argv));
+PushVertexList _ANSI_ARGS_((tSippGlob_pt    tSippGlobPtr,
+                            unsigned        options,
+                            char           *vertexList));
+
 /*=============================================================================
  * ClearPolygonStack --
  *    Clear the polygon stack if it contains any polygons.
@@ -61,8 +68,7 @@ ClearPolygonStack ()
 
     surfacePtr = surface_create (&surfDescHdr, NULL);
     if (surfacePtr != NULL)
-        surface_delete (surfacePtr);
-
+        surface_unref (surfacePtr);
 }
 
 /*=============================================================================
@@ -110,8 +116,7 @@ SurfaceHandleCleanup (tSippGlobPtr)
         if (surfaceEntryPtr == NULL)
             break;
 
-        surface_delete (*surfaceEntryPtr);  /* Decrements reference count */
-
+        surface_unref (*surfaceEntryPtr);
         Tcl_HandleFree (tSippGlobPtr->surfaceTblPtr, surfaceEntryPtr);
     }
 
@@ -144,39 +149,108 @@ TSippSurfaceHandleToPtr (tSippGlobPtr, handle)
 
 }
 
+/*=============================================================================
+ * ConvertVertexData --
+ *   Convert a list contain either vertex data.  This consists of the specified
+ * number of triples of numbers.
+ *
+ * Parameters:
+ *   o tSippGlobPtr (I) - A pointer to the Tcl SIPP global structure.
+ *   o listStr (I) - A Tcl list containing the vertex values to convert.
+ *   o numTriples (I) - Number of triples to expect.
+ *   o vertex (O) - Array that the data is returned in.
+ * Returns:
+ *   TRUE if the list and numbers are valid, FALSE if there is an error.
+ *-----------------------------------------------------------------------------
+ */
+static bool
+ConvertVertexData (tSippGlobPtr, listStr, numTriples, vertexData)
+    tSippGlob_pt  tSippGlobPtr;
+    char         *listStr;
+    int           numTriples;
+    Vector        vertexData [];
+{
+    int      dataArgc,  coordArgc, idx;
+    char   **dataArgv, **coordArgv;
+
+    if (Tcl_SplitList (tSippGlobPtr->interp, listStr,
+                       &dataArgc, &dataArgv) != TCL_OK)
+        return FALSE;
+
+    if (dataArgc != numTriples) {
+        char numStr [32];
+        sprintf (numStr, "%d", numTriples);
+        Tcl_AppendResult (tSippGlobPtr->interp, "data for each vertex ",
+                          "should consist of ", numStr, " sets of ",
+                          "coordinates or vectors, got \"", listStr, "\"",
+                          (char *) NULL);
+        goto errorCleanup;
+    }
+
+    for (idx = 0; idx < numTriples; idx++) {
+        if (Tcl_SplitList (tSippGlobPtr->interp, dataArgv [idx], &coordArgc,
+                           &coordArgv) != TCL_OK)
+            goto errorCleanup;
+
+        if (coordArgc != 3) {
+            Tcl_AppendResult (tSippGlobPtr->interp, "coordinate or vector ",
+                              " must be a list of three numbers, got \"",
+                              dataArgv [idx], "\"", (char *) NULL);
+            goto errorCleanup2;
+        }
+
+        if (Tcl_GetDouble (tSippGlobPtr->interp, coordArgv [0],
+                           &(vertexData [idx].x)) != TCL_OK)
+           goto errorCleanup2;
+        if (Tcl_GetDouble (tSippGlobPtr->interp, coordArgv [1],
+                           &(vertexData [idx].y)) != TCL_OK)
+           goto errorCleanup2;
+        if (Tcl_GetDouble (tSippGlobPtr->interp, coordArgv [2],
+                           &(vertexData [idx].z)) != TCL_OK)
+           goto errorCleanup2;
+
+        sfree (coordArgv);
+    }
+    sfree (dataArgv);
+    return TRUE;
+
+errorCleanup2:
+    sfree (coordArgv);
+errorCleanup:
+    sfree (dataArgv);
+    return FALSE;
+
+}
+
 /*=============================================================================
  * PushVertexList --
  *   Parse, convert a list of vertex coordinates, and optional texture
  * coordinates on the vertex stack.
  * Parameters:
  *   o tSippGlobPtr (I) - A pointer to the Tcl SIPP global structure.
- *   o textures (I) - TRUE if texture coordinates are includes, FALSE if not.
- *   o clockwise (I) - TRUE if vertices are clockwise, FALSE if counterclock-
- *     wise.
- *   o vertexList (I) - A list of lists of vertex coordinates or vertex and
- *     texture coordinates pairs.
+ *   o options (I) - Options describing the vertexes supplied.  
+ *   o vertexList (I) - A list of lists of vertex coordinates or a list of
+ *     vertex, normals and texture coordinates (in some combination).
  * Returns:
  *   TRUE if the list and numbers are valid, FALSE if there is an error.
- * Notes:
- *   There should be a way to clean up the vertex stack if an error occurs.
  *-----------------------------------------------------------------------------
  */
 static bool
-PushVertexList (tSippGlobPtr, textures, clockwise, vertexList)
+PushVertexList (tSippGlobPtr, options, vertexList)
     tSippGlob_pt    tSippGlobPtr;
-    bool            textures;
-    bool            clockwise;
+    unsigned        options;
     char           *vertexList;
 {
     int      vertexArgc, idx, idxLimit, idxIncr;
     char   **vertexArgv;
-    Vector   vertex, textureCoord;
+    Vector   vertex [3];
 
-    if (Tcl_SplitList (tSippGlobPtr->interp, vertexList, &vertexArgc,
+    if (Tcl_SplitList (tSippGlobPtr->interp,
+                       vertexList, &vertexArgc,
                        &vertexArgv) != TCL_OK)
         return FALSE;
 
-    if (clockwise) {
+    if (options & POLYGON_CLOCK) {
         idx      = vertexArgc - 1;
         idxLimit = -1;
         idxIncr  = -1;
@@ -186,21 +260,53 @@ PushVertexList (tSippGlobPtr, textures, clockwise, vertexList)
         idxIncr  = 1;
     }
 
+    switch (options & ~POLYGON_CLOCK) {
+      case POLYGON_NORM | POLYGON_TEX:
+        for (; idx != idxLimit; idx += idxIncr) {
+            if (!ConvertVertexData (tSippGlobPtr,
+                                    vertexArgv [idx],
+                                    3,
+                                    vertex))
+                goto errorExit;
+            vertex_tx_n_push (vertex [0].x, vertex [0].y, vertex [0].z,
+                              vertex [1].x, vertex [1].y, vertex [1].z,
+                              vertex [2].x, vertex [1].y, vertex [2].z);
+        }
+        break;
 
-    if (textures) {
+      case POLYGON_NORM:
         for (; idx != idxLimit; idx += idxIncr) {
-            if (!TSippConvertVertexTex (tSippGlobPtr, vertexArgv [idx],
-                                        &vertex, &textureCoord))
+            if (!ConvertVertexData (tSippGlobPtr,
+                                    vertexArgv [idx],
+                                    2,
+                                    vertex))
                 goto errorExit;
-            vertex_tx_push (vertex.x, vertex.y, vertex.z, 
-                            textureCoord.x, textureCoord.y, textureCoord.z);
+            vertex_n_push (vertex [0].x, vertex [0].y, vertex [0].z, 
+                           vertex [1].x, vertex [1].y, vertex [1].z);
         }
-    } else {
+        break;
+
+      case POLYGON_TEX:
         for (; idx != idxLimit; idx += idxIncr) {
-            if (!TSippConvertVertex (tSippGlobPtr, vertexArgv [idx], &vertex))
+            if (!ConvertVertexData (tSippGlobPtr,
+                                    vertexArgv [idx],
+                                    2,
+                                    vertex))
                 goto errorExit;
-            vertex_push (vertex.x, vertex.y, vertex.z);
+            vertex_tx_push (vertex [0].x, vertex [0].y, vertex [0].z, 
+                            vertex [1].x, vertex [1].y, vertex [1].z);
         }
+        break;
+
+      case 0:
+        for (; idx != idxLimit; idx += idxIncr) {
+            if (!TSippConvertVertex (tSippGlobPtr,
+                                     vertexArgv [idx],
+                                     &(vertex [0])))
+                goto errorExit;
+            vertex_push (vertex [0].x, vertex [0].y, vertex [0].z);
+        }
+        break;
     }
 
     sfree (vertexArgv);
@@ -211,7 +317,6 @@ PushVertexList (tSippGlobPtr, textures, clockwise, vertexList)
      * then clear the polygon stack.  This is a no-op if none have been pushed.
      */
 errorExit:
-    
     polygon_push ();
     ClearPolygonStack ();
 
@@ -237,9 +342,9 @@ SippPolygonPush (clientData, interp, argc, argv)
     char      **argv;
 {
     tSippGlob_pt  tSippGlobPtr = (tSippGlob_pt) clientData;
-    bool          textures, clockwise, gotTex, gotClock;
+    unsigned      options = 0, optionsSeen = 0;
     char         *flag;
-    int           argIdx;
+    int           argIdx = 1;
 
     if (tSippGlobPtr->rendering)
         return TSippNotWhileRendering (interp);
@@ -250,42 +355,52 @@ SippPolygonPush (clientData, interp, argc, argv)
     if (argc < 2)
         goto wrongArgs;
     
-    textures = clockwise = gotTex = gotClock = FALSE;
-    argIdx = 1;
-
     while ((argIdx < argc) && (argv [argIdx][0] == '-')) {
         flag = argv [argIdx];
         argIdx++;
-        if (STREQU (flag, "-tex")) {
-            if (gotTex)
+        if (STREQU (flag, "-norm")) {
+            if (optionsSeen & POLYGON_NORM)
+                goto dupNorm;
+            options |= POLYGON_NORM;
+            optionsSeen |= POLYGON_NORM;
+            continue;
+        }
+        if (STREQU (flag, "-nonorm")) {
+            if (optionsSeen & POLYGON_NORM)
                 goto dupTex;
-            gotTex = TRUE;
-            textures = TRUE;
+            optionsSeen |= POLYGON_NORM;
+            continue;
+        }
+        if (STREQU (flag, "-tex")) {
+            if (optionsSeen & POLYGON_TEX)
+                goto dupTex;
+            options |= POLYGON_TEX;
+            optionsSeen |= POLYGON_TEX;
             continue;
         }
         if (STREQU (flag, "-notex")) {
-            if (gotTex)
+            if (optionsSeen & POLYGON_TEX)
                 goto dupTex;
-            gotTex = TRUE;
-            textures = FALSE;
+            optionsSeen |= POLYGON_TEX;
             continue;
         }
         if (STREQU (flag, "-clock")) {
-            if (gotClock)
+            if (options & POLYGON_CLOCK)
                 goto dupClock;
-            gotClock = TRUE;
-            clockwise = TRUE;
+            options |= POLYGON_CLOCK;
+            optionsSeen |= POLYGON_CLOCK;
             continue;
         }
         if (STREQU (flag, "-counter")) {
-            if (gotClock)
+            if (options & POLYGON_CLOCK)
                 goto dupClock;
-            gotClock = TRUE;
-            clockwise = FALSE;
+            optionsSeen |= POLYGON_CLOCK;
             continue;
         }
         Tcl_AppendResult (interp, "invalid flag \"", flag,
-                          " expected one of:  \"-tex\", \"-notex\" ",
+                          "\" expected one of: ",
+                          "\"-norm\", \"-nonorm\", ",
+                          "\"-tex\", \"-notex\", ",
                           "\"-clock\", or \"-counter\"",
                           (char *) NULL);
         return TCL_ERROR;
@@ -294,7 +409,7 @@ SippPolygonPush (clientData, interp, argc, argv)
     if (argIdx != argc - 1)
         goto wrongArgs;
 
-    if (!PushVertexList (tSippGlobPtr, textures, clockwise, argv [argIdx]))
+    if (!PushVertexList (tSippGlobPtr, options, argv [argIdx]))
         return TCL_ERROR;
 
     polygon_push ();
@@ -305,6 +420,10 @@ SippPolygonPush (clientData, interp, argc, argv)
                       " [-flags] vertexList", (char *) NULL);
     return TCL_ERROR;
 
+  dupNorm:
+    interp->result = "can only specify one of \"-norm\" or \"-nonorm\"";
+    return TCL_ERROR;
+
   dupTex:
     interp->result = "can only specify one of \"-tex\" or \"-notex\"";
     return TCL_ERROR;
@@ -312,8 +431,6 @@ SippPolygonPush (clientData, interp, argc, argv)
   dupClock:
     interp->result = "can only specify one of \"-clock\" or \"-counter\"";
     return TCL_ERROR;
-
-
 }
 
 /*=============================================================================
@@ -436,7 +553,7 @@ SippSurfaceUnref (clientData, interp, argc, argv)
         return TCL_ERROR;
 
     for (idx = 0; idx < surfaceList.len; idx++) {
-        surface_delete (surfaceList.ptr [idx]);  /* Decrements ref count */
+        surface_unref (surfaceList.ptr [idx]);
         Tcl_HandleFree (tSippGlobPtr->surfaceTblPtr, 
                         surfaceEntryList.ptr [idx]);
     }
