@@ -11,12 +11,30 @@
  * no representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tSippPrim.c,v 3.0 1993/01/20 06:43:42 markd Rel $
+ * $Id: tSippPrim.c,v 5.0 1994/09/05 01:23:25 markd Rel $
  *=============================================================================
  */
 
 #include "tSippInt.h"
 
+/*
+ * Prototype of function used to generate teapot parts.
+ */
+typedef Object *(teapotFunc_t) _ANSI_ARGS_((int            resolution,
+                                            void          *surface,
+                                            Shader        *shader,
+                                            int            texture));
+
+/*
+ * Internal prototypes.
+ */
+static int
+GenerateTeapot _ANSI_ARGS_((tSippGlob_pt   tSippGlobPtr,
+                            Tcl_Interp    *interp,
+                            int            argc,
+                            char         **argv,
+                            teapotFunc_t   teapotFunc));
+
 /*=============================================================================
  * SippTorus --
  *   Implements the command:
@@ -503,6 +521,170 @@ SippCube (clientData, interp, argc, argv)
 
 }
 
+/*=============================================================================
+ * GenerateTeapot --
+ *   
+ *  Command executor for all of the SippTeapot* commands.
+ *
+ * Parameters:
+ *   o tSippGlobPtr (I) - A pointer to the Tcl SIPP global structure.
+ *   o interp (I) - Errors are returned in result.
+ *   o argc, argv (I) - Command arguments.
+ *   o teapotFunc (I) - SIPP function to call to generate the teapot.
+ * Returns:
+ *   TCL_OK or TCL_ERROR;
+ *-----------------------------------------------------------------------------
+ */
+static int
+GenerateTeapot (tSippGlobPtr, interp, argc, argv, teapotFunc)
+    tSippGlob_pt   tSippGlobPtr;
+    Tcl_Interp    *interp;
+    int            argc;
+    char         **argv;
+    teapotFunc_t   teapotFunc;
+{
+    Shader        *shaderPtr;
+    void          *surfDescPtr;
+    unsigned       resolution;
+    int            texture = NATURAL;
+
+    if (tSippGlobPtr->rendering)
+        return TSippNotWhileRendering (interp);
+
+    if ((argc < 3) || (argc > 4)) {
+        Tcl_AppendResult (interp, "wrong # args: ", argv [0],
+                          " resolution shaderhandle [texture]", (char *) NULL);
+        return TCL_ERROR;
+    }                     
+    if (!TSippConvertPosUnsigned (tSippGlobPtr, argv [1], &resolution))
+        return TCL_ERROR;
+
+    shaderPtr = TSippShaderHandleToPtr (tSippGlobPtr, argv [2],
+                                        &surfDescPtr);
+    if (shaderPtr == NULL)
+        return TCL_ERROR;
+
+    if (argc == 4) {
+        if (!TSippParseTextureMapping (tSippGlobPtr, argv [3], &texture, NULL))
+            return TCL_ERROR;
+    }
+    TSippBindObjectToHandle (tSippGlobPtr, 
+                             teapotFunc (resolution, surfDescPtr, shaderPtr,
+                                         texture)); 
+    return TCL_OK;
+
+}
+
+/*=============================================================================
+ * SippTeapot --
+ *   Implements the command:
+ *      SippTeapot resolution shaderhandle [texture]
+ * Note:
+ *   This procedure has standard Tcl command calling sematics.  ClientData
+ * contains a pointer to the Tcl SIPP global structure.
+ *-----------------------------------------------------------------------------
+ */
+int
+SippTeapot (clientData, interp, argc, argv)
+    char       *clientData;
+    Tcl_Interp *interp;
+    int         argc;
+    char      **argv;
+{
+    return GenerateTeapot ((tSippGlob_pt) clientData,
+                           interp,
+                           argc, argv,
+                           sipp_teapot);
+}
+
+/*=============================================================================
+ * SippTeapotBody --
+ *   Implements the command:
+ *      SippTeapotBody resolution shaderhandle [texture]
+ * Note:
+ *   This procedure has standard Tcl command calling sematics.  ClientData
+ * contains a pointer to the Tcl SIPP global structure.
+ *-----------------------------------------------------------------------------
+ */
+int
+SippTeapotBody (clientData, interp, argc, argv)
+    char       *clientData;
+    Tcl_Interp *interp;
+    int         argc;
+    char      **argv;
+{
+    return GenerateTeapot ((tSippGlob_pt) clientData,
+                           interp,
+                           argc, argv,
+                           sipp_teapot_body);
+}
+
+/*=============================================================================
+ * SippTeapotLid --
+ *   Implements the command:
+ *      SippTeapotLid resolution shaderhandle [texture]
+ * Note:
+ *   This procedure has standard Tcl command calling sematics.  ClientData
+ * contains a pointer to the Tcl SIPP global structure.
+ *-----------------------------------------------------------------------------
+ */
+int
+SippTeapotLid (clientData, interp, argc, argv)
+    char       *clientData;
+    Tcl_Interp *interp;
+    int         argc;
+    char      **argv;
+{
+    return GenerateTeapot ((tSippGlob_pt) clientData,
+                           interp,
+                           argc, argv,
+                           sipp_teapot_lid);
+}
+
+/*=============================================================================
+ * SippTeapotHandle --
+ *   Implements the command:
+ *      SippTeapotHandle resolution shaderhandle [texture]
+ * Note:
+ *   This procedure has standard Tcl command calling sematics.  ClientData
+ * contains a pointer to the Tcl SIPP global structure.
+ *-----------------------------------------------------------------------------
+ */
+int
+SippTeapotHandle (clientData, interp, argc, argv)
+    char       *clientData;
+    Tcl_Interp *interp;
+    int         argc;
+    char      **argv;
+{
+    return GenerateTeapot ((tSippGlob_pt) clientData,
+                           interp,
+                           argc, argv,
+                           sipp_teapot_handle);
+}
+
+/*=============================================================================
+ * SippTeapotSpout --
+ *   Implements the command:
+ *      SippTeapotSpout resolution shaderhandle [texture]
+ * Note:
+ *   This procedure has standard Tcl command calling sematics.  ClientData
+ * contains a pointer to the Tcl SIPP global structure.
+ *-----------------------------------------------------------------------------
+ */
+int
+SippTeapotSpout (clientData, interp, argc, argv)
+    char       *clientData;
+    Tcl_Interp *interp;
+    int         argc;
+    char      **argv;
+{
+    return GenerateTeapot ((tSippGlob_pt) clientData,
+                           interp,
+                           argc, argv,
+                           sipp_teapot_spout);
+}
+
 /*=============================================================================
  * TSippPrimInit --
  *   Initialized the primitve object creation commands.
@@ -516,19 +698,23 @@ TSippPrimInit (tSippGlobPtr)
     tSippGlob_pt  tSippGlobPtr;
 {
     static tSippTclCmdTbl_t cmdTable [] = {
-        {"SippTorus",      (Tcl_CmdProc *) SippTorus},
-        {"SippCone",       (Tcl_CmdProc *) SippCone},
-        {"SippCylinder",   (Tcl_CmdProc *) SippCylinder},
-        {"SippEllipsoid",  (Tcl_CmdProc *) SippEllipsoid},
-        {"SippSphere",     (Tcl_CmdProc *) SippSphere},
-        {"SippPrism",      (Tcl_CmdProc *) SippPrism},
-        {"SippBlock",      (Tcl_CmdProc *) SippBlock},
-        {"SippCube",       (Tcl_CmdProc *) SippCube},
-        {NULL,             NULL}
+        {"SippTorus",        (Tcl_CmdProc *) SippTorus},
+        {"SippCone",         (Tcl_CmdProc *) SippCone},
+        {"SippCylinder",     (Tcl_CmdProc *) SippCylinder},
+        {"SippEllipsoid",    (Tcl_CmdProc *) SippEllipsoid},
+        {"SippSphere",       (Tcl_CmdProc *) SippSphere},
+        {"SippPrism",        (Tcl_CmdProc *) SippPrism},
+        {"SippBlock",        (Tcl_CmdProc *) SippBlock},
+        {"SippCube",         (Tcl_CmdProc *) SippCube},
+        {"SippTeapot",       (Tcl_CmdProc *) SippTeapot},
+        {"SippTeapotBody",   (Tcl_CmdProc *) SippTeapotBody},
+        {"SippTeapotLid",    (Tcl_CmdProc *) SippTeapotLid},
+        {"SippTeapotHandle", (Tcl_CmdProc *) SippTeapotHandle},
+        {"SippTeapotSpout",  (Tcl_CmdProc *) SippTeapotSpout},
+        {NULL,                NULL}
     };
 
     TSippInitCmds (tSippGlobPtr, cmdTable);
-
 }
 
 
