@@ -11,10 +11,10 @@
  * no representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  *-----------------------------------------------------------------------------
- * Out own functions are used to manage comments to prevent memory lose that
- * occurs in the RLE library.
+ * Our own functions are used to manage comments and other aspects of the RLE
+ * header to prevent memory lose that occurs in the RLE library.
  *-----------------------------------------------------------------------------
- * $Id: tSippRLE.c,v 3.0 1993/01/20 06:43:44 markd Rel $
+ * $Id: tSippRLE.c,v 5.0 1994/09/05 01:23:28 markd Rel $
  *=============================================================================
  */
 
@@ -22,23 +22,32 @@
 #include "sipp_pixmap.h"
 #include "sipp_bitmap.h"
 
-#ifndef TSIPP_NO_RLE
+#ifdef TSIPP_HAVE_RLE
 
 #include "rle.h"
 
 /*
- * Type used to hold a file handle "fileNNN".
+ * Check if this is the 3.1 beta release of URT.
+ */
+#ifdef RLE_INIT_MAGIC
+#   define URT_3_1b
+#endif
+
+
+/*
+ * Type used to hold a file handle "fileNNN" or RLE file handle "rleNNN".
  */
 typedef char fileHandle_t [10];
 
 /*
- * RLE file table entry.
+ * RLE file table entry.  Note that name is redundent with RLE 3.1, as its
+ * in the RLE header.
  */
 typedef struct {
     fileHandle_t  handle;
     FILE         *filePtr;
-    bool          readable;
-    bool          writable;
+    char         *name;
+    int           perms;
     bool          eof;
     rle_hdr       rleHeader;
 } rleFile_t, *rleFile_pt;
@@ -58,21 +67,25 @@ typedef struct {
 /*
  * Internal prototypes.
  */
-static void
-RLECleanHeader _ANSI_ARGS_((rle_hdr *rleHdrPtr,
-                            bool     cleanComments));
-
 static rleFile_pt
 RLEHandleToPtr _ANSI_ARGS_((tSippGlob_pt    tSippGlobPtr,
                             char           *handle));
 
 static void
-InitRLEHeader _ANSI_ARGS_((rleFile_pt    rleFilePtr));
+RLEInitHeader _ANSI_ARGS_((Tcl_Interp *interp,
+                           FILE       *filePtr,
+                           char       *fileName,
+                           rle_hdr    *rleHdrPtr));
+
+static void
+RLEFreeHeader _ANSI_ARGS_((rle_hdr *rleHdrPtr));
+
+static void
+RLEResetHeader _ANSI_ARGS_((rle_hdr *rleHdrPtr));
 
 static bool
 ReadRLEHeader _ANSI_ARGS_((tSippGlob_pt  tSippGlobPtr,
-                           rleFile_pt    rleFilePtr,
-                           char         *name));
+                           rleFile_pt    rleFilePtr));
 
 static void
 CopyRLEHeader _ANSI_ARGS_((rleFile_pt    srcFilePtr,
@@ -137,35 +150,6 @@ RLECopyImage _ANSI_ARGS_((tSippGlob_pt          tSippGlobPtr,
 static void
 RLECleanUp _ANSI_ARGS_((tSippGlob_pt  tSippGlobPtr));
 
-/*=============================================================================
- * RLECleanHeader --
- *   Free all dynamic memory associated with an RLE header.  This does not
- * free the header itself.
- * 
- * Parameters:
- *   o rleHdrPtr (I) - Pointer to the RLE file structure.
- *   o cleanComments (I) - If TRUE, comments are reset, if FALSE they are
- *     left alone.
- *-----------------------------------------------------------------------------
- */
-static void
-RLECleanHeader (rleHdrPtr, cleanComments)
-    rle_hdr *rleHdrPtr;
-    bool     cleanComments;
-{
-    if (rleHdrPtr->cmap != NULL) {
-        sfree (rleHdrPtr->cmap);
-        rleHdrPtr->cmap = NULL;
-    }
-    if (rleHdrPtr->bg_color != NULL) {
-        sfree (rleHdrPtr->bg_color);
-        rleHdrPtr->bg_color = NULL;
-    }
-    if (cleanComments)
-        TSippFreeCom (&rleHdrPtr->comments);
-
-}
-
 /*=============================================================================
  * RLEHandleToPtr --
  *   Utility procedure to convert a RLE file handle to a RLE header pointer.
@@ -193,40 +177,109 @@ RLEHandleToPtr (tSippGlobPtr, handle)
 }
 
 /*=============================================================================
- * InitRLEHeader --
- *   Initialize  the header of a RLE file.  This is used to setup a header for
- * a file that is to be written.  If the header had data in it, RLECleanHeader
- * should be called before this function.  This function resets dynamic info
- * in the rleFilePtr
+ * RLEInitHeader --
+ *   Initialize an RLE header for a just opened file.  Care is taken to set up
+ * the RLE header so it doesn't leak memory.
  * 
  * Parameters:
- *   o rleFilePtr (I) - Pointer to the RLE file structure containing the
- *     header.  The filePtr field must be initialized.
+ *   o interp (I) - Pointer to the Tcl interpreter, used to get argv0.
+ *   o filePtr (I) - Pointer to the file that was opened.
+ *   o fileName (I) - Name the file was opened.
+ *   o rleHdrPtr (O) - Pointer to the RLE header to initialize.
  *-----------------------------------------------------------------------------
  */
 static void
-InitRLEHeader (rleFilePtr)
-    rleFile_pt    rleFilePtr;
+RLEInitHeader (interp, filePtr, fileName, rleHdrPtr)
+    Tcl_Interp *interp;
+    FILE       *filePtr;
+    char       *fileName;
+    rle_hdr    *rleHdrPtr;
 {
-    rleFilePtr->eof = TRUE;  /* Assume writing at the end. */
+#ifdef URT_3_1b
+    char *programName;
 
-    rleFilePtr->rleHeader = rle_dflt_hdr;
-    rleFilePtr->rleHeader.rle_file = rleFilePtr->filePtr;
+    /*
+     * Get the program name to store in the header.
+     */
+    programName = Tcl_GetVar (interp, "argv0", TCL_GLOBAL_ONLY);
+    if (programName == NULL)
+        programName = tclAppName;
+
+    /*
+     * Init header. Make sure magic check doesn't succeed.
+     */
+    rleHdrPtr->is_init = 0;
+    *rleHdrPtr = *rle_hdr_init (NULL);
+
+    /*
+     * Delete the names init put in and put in our own.
+     */
+    sfree ((char *) rleHdrPtr->cmd);
+    rleHdrPtr->cmd = NULL;
+    sfree ((char *) rleHdrPtr->file_name);
+    rleHdrPtr->file_name = NULL;
+    rle_names (rleHdrPtr, programName, fileName, 0);
+#else
+    *rleHdrPtr = rle_dflt_hdr;
+    rleHdrPtr->bg_color = NULL;
+#endif
+    rleHdrPtr->rle_file = filePtr;
 
     /*
      * Indicate that an image is zero sized until specified by reading,
-     * rendering or copying.
+     * rendering or copying.  ?????
      */
-    rleFilePtr->rleHeader.xmin = 0;
-    rleFilePtr->rleHeader.xmax = -1;
-    rleFilePtr->rleHeader.ymin = 0;
-    rleFilePtr->rleHeader.ymax = -1;
+    rleHdrPtr->xmin =  0;
+    rleHdrPtr->xmax = -1;
+    rleHdrPtr->ymin =  0;
+    rleHdrPtr->ymax = -1;
+}
 
-    /*
-     * Allocate room for background RGB values.
-     */
-    rleFilePtr->rleHeader.bg_color = (int *) smalloc (3 * sizeof (int));
+/*=============================================================================
+ * RLEFreeHeader --
+ *   Free all dynamic memory associated with an RLE header before the file is
+ * closed.  
+ * 
+ * Parameters:
+ *   o rleHdrPtr (I) - Pointer to the RLE header.  The header structure is
+ *     not freed, only the dynamic contents.
+ *-----------------------------------------------------------------------------
+ */
+static void
+RLEFreeHeader (rleHdrPtr)
+    rle_hdr *rleHdrPtr;
+{
+    RLEResetHeader (rleHdrPtr);
 
+    if (rleHdrPtr->cmd != NULL)
+        sfree ((char *) rleHdrPtr->cmd);
+    if (rleHdrPtr->file_name != NULL)
+        sfree ((char *) rleHdrPtr->file_name);
+}
+
+/*=============================================================================
+ * RLEResetHeader --
+ *   Free all dynamic memory associated with an RLE header.  This does not
+ * free the header itself or change the file name in the header.
+ * 
+ * Parameters:
+ *   o rleHdrPtr (I) - Pointer to the RLE header.
+ *-----------------------------------------------------------------------------
+ */
+static void
+RLEResetHeader (rleHdrPtr)
+    rle_hdr *rleHdrPtr;
+{
+    if (rleHdrPtr->cmap != NULL) {
+        sfree (rleHdrPtr->cmap);
+        rleHdrPtr->cmap = NULL;
+    }
+    if (rleHdrPtr->bg_color != NULL) {
+        sfree (rleHdrPtr->bg_color);
+        rleHdrPtr->bg_color = NULL;
+    }
+    if (rleHdrPtr->comments != NULL)
+        TSippFreeCom ((char ***) &rleHdrPtr->comments);
 }
 
 /*=============================================================================
@@ -240,54 +293,57 @@ InitRLEHeader (rleFilePtr)
  *   o tSippGlobPtr (I) - A pointer to the Tcl SIPP global structure.
  *   o rleFilePtr (I) - Pointer to the RLE file structure containing the
  *     header.  The filePtr field must be initialized.
- *   o name (I) - Name to indentify the file in an error message.  This is
- *     either the file name or handle.
  * Returns:
  *   TRUE if all is OK, FALSE if an error occured with error message in
  *   tSippGlobPtr->interp->result.
  *-----------------------------------------------------------------------------
  */
 static bool
-ReadRLEHeader (tSippGlobPtr, rleFilePtr, name)
+ReadRLEHeader (tSippGlobPtr, rleFilePtr)
     tSippGlob_pt  tSippGlobPtr;
     rleFile_pt    rleFilePtr;
-    char         *name;
 {
     int    status;
     char **oldArrayPtr;
 
-    RLECleanHeader (&rleFilePtr->rleHeader, TRUE);  /* Clean comments */
+    RLEResetHeader (&rleFilePtr->rleHeader);
     rleFilePtr->rleHeader.rle_file = rleFilePtr->filePtr;
 
     status = rle_get_setup (&rleFilePtr->rleHeader);
 
-    /*
-     * Convert comments to standard malloced string layout.
-     */
-    if (rleFilePtr->rleHeader.comments != NULL) {
-        oldArrayPtr = rleFilePtr->rleHeader.comments;
-        rleFilePtr->rleHeader.comments = TSippCopyCom (oldArrayPtr);
-        sfree (oldArrayPtr);
-    }
-
     switch (status) {
       case RLE_SUCCESS:
         rleFilePtr->eof = FALSE;
-        return TRUE;
+        break;
       case RLE_EMPTY:
       case RLE_EOF:
         rleFilePtr->eof = TRUE;
-        return TRUE;
+        break;
       case RLE_NOT_RLE:
-        Tcl_AppendResult (tSippGlobPtr->interp, name, 
+        Tcl_AppendResult (tSippGlobPtr->interp, rleFilePtr->name,
                           " is an invalid RLE file", (char *) NULL);
         return FALSE;
       case RLE_NO_SPACE:
         panic ("rle_get_setup: malloc failed");
     }
 
-    return FALSE;  /* Should never make it here */
+    /*???Check for useable RLE file (i.e. 3 colors, etc).
+     * Handle color maps (limited).
+     */
 
+    /*
+     * Convert comments to standard malloced string layout.  This is really
+     * dangerous.  I knows how rle_get_setup works.
+     */
+    if (rleFilePtr->rleHeader.comments != NULL) {
+        oldArrayPtr = (char **) rleFilePtr->rleHeader.comments;
+        rleFilePtr->rleHeader.comments = 
+            (CONST_DECL char **) TSippCopyCom (oldArrayPtr);
+        sfree (oldArrayPtr [0]);
+        sfree (oldArrayPtr);
+    }
+
+    return TRUE;
 }
 
 /*=============================================================================
@@ -312,7 +368,7 @@ CopyRLEHeader (srcFilePtr, destFilePtr, mergeComments)
     bool          mergeComments;
 {
     int    sizeBytes;
-    char **saveComments;
+    char **saveComments = NULL, *holdCmd, *holdFName;
 
     /*
      * Cleanup the header then copy all of the static fields.  The only thing
@@ -321,12 +377,31 @@ CopyRLEHeader (srcFilePtr, destFilePtr, mergeComments)
      * merged.  The comment pointer saving code just copies a NULL if
      * mergeComments is FALSE;
      */
-    RLECleanHeader (&destFilePtr->rleHeader, mergeComments == 0);
+    if (mergeComments) {
+        saveComments = (char **) destFilePtr->rleHeader.comments;
+        destFilePtr->rleHeader.comments = NULL;
+    }
+
+    RLEResetHeader (&destFilePtr->rleHeader);
+
+    /*
+     * Copy the header.  We don't even bother with rle_hdr_cp, as it loses
+     * too much memory.
+     */
+#ifdef URT_3_1b
+    holdCmd = (char *) destFilePtr->rleHeader.cmd;
+    holdFName = (char *) destFilePtr->rleHeader.file_name;
 
-    saveComments = destFilePtr->rleHeader.comments;
     destFilePtr->rleHeader = srcFilePtr->rleHeader;
-    destFilePtr->rleHeader.rle_file = destFilePtr->filePtr;
+
+    destFilePtr->rleHeader.cmd = holdCmd;
+    destFilePtr->rleHeader.file_name = holdFName;
+    destFilePtr->rleHeader.comments = (CONST_DECL char **) saveComments;
+#else
+    destFilePtr->rleHeader = srcFilePtr->rleHeader;
     destFilePtr->rleHeader.comments = saveComments;
+#endif
+    destFilePtr->rleHeader.rle_file = destFilePtr->filePtr;
 
     /*
      * Copy the color map
@@ -354,11 +429,11 @@ CopyRLEHeader (srcFilePtr, destFilePtr, mergeComments)
      * Copy or merge the comments.
      */
     if (mergeComments)
-        TSippMergeCom (srcFilePtr->rleHeader.comments,
-                       &destFilePtr->rleHeader.comments);
+        TSippMergeCom ((char **) srcFilePtr->rleHeader.comments,
+                       (char ***) &destFilePtr->rleHeader.comments);
     else
-        destFilePtr->rleHeader.comments =
-            TSippCopyCom (srcFilePtr->rleHeader.comments);
+        destFilePtr->rleHeader.comments = (CONST_DECL char **) 
+            TSippCopyCom ((char **) srcFilePtr->rleHeader.comments);
 
 }
 
@@ -381,18 +456,20 @@ CloseRLEFile (tSippGlobPtr, command, rleFilePtr)
     rleFile_pt      rleFilePtr;
 {
     int             result;
-    char           *closeArgv [2];
+    char           *closeArgv [3];
 
     closeArgv [0] = command;
     closeArgv [1] = rleFilePtr->handle;
+    closeArgv [2] = NULL;
 
     result = Tcl_CloseCmd (NULL, tSippGlobPtr->interp, 2, closeArgv);
 
-    RLECleanHeader (&rleFilePtr->rleHeader, TRUE);  /* Clean comments */
+    RLEFreeHeader (&rleFilePtr->rleHeader);
 
+    sfree (rleFilePtr->name);
     Tcl_HandleFree (tSippGlobPtr->rleTblPtr, rleFilePtr);
-    return (result == TCL_OK);
 
+    return (result == TCL_OK);
 }
 
 /*=============================================================================
@@ -414,9 +491,10 @@ SippRLEOpen (clientData, interp, argc, argv)
     char      **argv;
 {
     tSippGlob_pt    tSippGlobPtr = (tSippGlob_pt) clientData;
-    fileHandle_t    fileHandle;
+    fileHandle_t    fileHandle, rleHandle;
     rleFile_pt      rleFilePtr;
-    OpenFile       *filePtr;
+    FILE           *filePtr;
+    int             perms;
     char           *errorMsg;
 
     if (tSippGlobPtr->rendering)
@@ -438,45 +516,45 @@ SippRLEOpen (clientData, interp, argc, argv)
     strcpy (fileHandle, interp->result);
     Tcl_ResetResult (interp);
 
-    if (TclGetOpenFile (interp, fileHandle, &filePtr) != TCL_OK)
+    if (Tcl_GetOpenFile (interp, fileHandle, FALSE, FALSE, &filePtr) != TCL_OK)
          return TCL_ERROR;  /* Should never happen */
 
+    perms = Tcl_FilePermissions (filePtr);
+
+    /*
+     * Allocate and initialize the RLE file structure and the RLE header.
+     */
+    rleFilePtr = (rleFile_pt) Tcl_HandleAlloc (tSippGlobPtr->rleTblPtr,
+                                               rleHandle);
+    strcpy (rleFilePtr->handle, fileHandle);
+    rleFilePtr->name = strdup (argv [1]);
+    rleFilePtr->perms = perms;
+    rleFilePtr->filePtr = filePtr;
+    rleFilePtr->eof = TRUE;  /* Assume writing at the end. */
+
+    RLEInitHeader (interp, filePtr, argv [1], &rleFilePtr->rleHeader);
+
     /*
      * Make sure that the file will only be opened for reading or writting,
-     * but not both.
+     * but not both.  This is done after the header is setup so that we 
+     * can use standard error close path.
      */
-    if (filePtr->readable && filePtr->writable) {
+    if ((perms & TCL_FILE_READABLE) && (perms & TCL_FILE_WRITABLE)) {
         Tcl_AppendResult (interp, "RLE files may be opened for reading or ",
                           "writing, but not both", (char *) NULL);
         goto errorExit;
     }
 
-    /*
-     * Allocate and initialize the RLE header struct, defaulting all fields.
-     * If its open for read access, then read the header.
-     */
-    rleFilePtr = (rleFile_pt) Tcl_HandleAlloc (tSippGlobPtr->rleTblPtr,
-                                               interp->result);
-    strcpy (rleFilePtr->handle, fileHandle);
-    rleFilePtr->readable  = filePtr->readable;
-    rleFilePtr->writable  = filePtr->writable;
-    rleFilePtr->filePtr   = filePtr->f;
-
-    InitRLEHeader (rleFilePtr);
-
     /*
      * If read access, get the header.  Must stash the handle in case an
      * error gets generated.
      */
-    strcpy (fileHandle, interp->result);
-    Tcl_ResetResult (interp);
-
-    if (rleFilePtr->readable) {
-        if (!ReadRLEHeader (tSippGlobPtr, rleFilePtr, fileHandle))
+    if (rleFilePtr->perms & TCL_FILE_READABLE) {
+        if (!ReadRLEHeader (tSippGlobPtr, rleFilePtr))
             goto errorExit;
     }
 
-    strcpy (interp->result, fileHandle);
+    strcpy (interp->result, rleHandle);
     return TCL_OK;
 
     /*
@@ -568,11 +646,12 @@ RLECommentCmdSetup (tSippGlobPtr, handle, writeComment, name)
      * Check access.  You can read something that is write only, as the
      * header is stashed in memory.
      */
-    if (writeComment & !rleFilePtr->writable) {
+    if (writeComment && ((rleFilePtr->perms & TCL_FILE_WRITABLE) == 0)) {
         TSippNotWritable (tSippGlobPtr->interp, name);
         return NULL;
     }
-    if (!writeComment & !rleFilePtr->writable & rleFilePtr->eof) {
+    if (!writeComment && rleFilePtr->eof &&
+        ((rleFilePtr->perms & TCL_FILE_WRITABLE) == 0)) {
         TSippAtEOF (tSippGlobPtr->interp, name);
         return NULL;
     }
@@ -617,7 +696,8 @@ SippRLEPutCom (clientData, interp, argc, argv)
     if (rleFilePtr == NULL)
         return TCL_ERROR;
 
-    TSippPutCom (&rleFilePtr->rleHeader.comments, argv [2], argv [3]);
+    TSippPutCom ((char ***) &rleFilePtr->rleHeader.comments,
+                 argv [2], argv [3]);
 
     return TCL_OK;
 
@@ -661,8 +741,8 @@ SippRLEGetCom (clientData, interp, argc, argv)
      */
     if (argc == 2) {
         Tcl_SetResult (interp,
-                       TSippGetAllCom (rleFilePtr->rleHeader.comments),
-                       TCL_DYNAMIC);
+                    TSippGetAllCom ((char **) rleFilePtr->rleHeader.comments),
+                    TCL_DYNAMIC);
         return TCL_OK;
     }
 
@@ -671,7 +751,7 @@ SippRLEGetCom (clientData, interp, argc, argv)
      * false or leave result an empty string.  If its was found, either return
      * it in the variable or as the command result.
      */
-    comStr = TSippGetCom (rleFilePtr->rleHeader.comments, argv [2]);
+    comStr = TSippGetCom ((char **) rleFilePtr->rleHeader.comments, argv [2]);
 
     if (comStr == NULL) {
         if (argc == 4)
@@ -725,9 +805,9 @@ SippRLEDelCom (clientData, interp, argc, argv)
         return TCL_ERROR;
 
     if (argc == 2)
-        TSippFreeCom (&rleFilePtr->rleHeader.comments);
+        TSippFreeCom ((char ***) &rleFilePtr->rleHeader.comments);
     else
-        TSippDelCom (&rleFilePtr->rleHeader.comments, argv [2]);
+        TSippDelCom ((char ***) &rleFilePtr->rleHeader.comments, argv [2]);
 
     return TCL_OK;
 
@@ -824,13 +904,13 @@ RLEOutputStart (tSippGlobPtr, outputParmsPtr, handle, comments)
     if (rleFilePtr == NULL)
         return NULL;
 
-    if (!rleFilePtr->writable) {
+    if ((rleFilePtr->perms & TCL_FILE_WRITABLE) == 0) {
         TSippNotWritable (tSippGlobPtr->interp, handle);
         return NULL;
     }
 
     if (comments != NULL) {
-        TSippMergeCom (comments, &rleFilePtr->rleHeader.comments);
+        TSippMergeCom (comments, (char ***) &rleFilePtr->rleHeader.comments);
     }
 
     /*
@@ -846,6 +926,8 @@ RLEOutputStart (tSippGlobPtr, outputParmsPtr, handle, comments)
     RLE_SET_BIT (rleFilePtr->rleHeader, RLE_GREEN);
     RLE_SET_BIT (rleFilePtr->rleHeader, RLE_BLUE);
 
+    if (rleFilePtr->rleHeader.bg_color == NULL)
+        rleFilePtr->rleHeader.bg_color = (int *) smalloc (3 * sizeof (int));
     rleFilePtr->rleHeader.bg_color [RLE_RED] =
         tSippGlobPtr->backgroundColor.red * 255;
     rleFilePtr->rleHeader.bg_color [RLE_GREEN] =
@@ -858,7 +940,6 @@ RLEOutputStart (tSippGlobPtr, outputParmsPtr, handle, comments)
     /*
      * Write the header to file.
      */
-
     rle_put_setup (&rleFilePtr->rleHeader);
 
     /*
@@ -995,8 +1076,7 @@ RLEOutputEnd (tSippGlobPtr, outputParmsPtr, renderDataPtr)
    /*
     * Reset the output file header, since we will start with a new image.
     */
-   RLECleanHeader (renderDataPtr->rleHdrPtr, TRUE); /* Clean comments */
-   InitRLEHeader (renderDataPtr->rleFilePtr);
+   RLEResetHeader (renderDataPtr->rleHdrPtr);
 
    return TRUE;
 
@@ -1029,7 +1109,8 @@ ParseImageComment (tSippGlobPtr, rleFilePtr, outputParmsPtr)
     int    infoArgc;
     Color  backgroundColor, lineColor;
 
-    infoList = TSippGetCom (rleFilePtr->rleHeader.comments, "TSIPP_IMAGE");
+    infoList = TSippGetCom ((char **) rleFilePtr->rleHeader.comments,
+                            "TSIPP_IMAGE");
     if (infoList == NULL)
         return TRUE;
 
@@ -1150,7 +1231,7 @@ RLEToRLECopy (tSippGlobPtr, srcHandle, srcFilePtr, destHandle)
     if (destFilePtr == NULL)
         return FALSE;
 
-    if (!destFilePtr->writable) {
+    if ((destFilePtr->perms & TCL_FILE_WRITABLE) == 0) {
         TSippNotWritable (tSippGlobPtr->interp, destHandle);
         return FALSE;
     }
@@ -1158,8 +1239,7 @@ RLEToRLECopy (tSippGlobPtr, srcHandle, srcFilePtr, destHandle)
     /*
      * Copy the header and data.
      */
-
-    CopyRLEHeader (srcFilePtr, destFilePtr, TRUE);  /* Merge the headers */
+    CopyRLEHeader (srcFilePtr, destFilePtr, TRUE);  /* Merge comments */
     
     rle_put_setup (&destFilePtr->rleHeader);
 
@@ -1168,13 +1248,10 @@ RLEToRLECopy (tSippGlobPtr, srcHandle, srcFilePtr, destHandle)
     fflush (destFilePtr->filePtr);
 
     /*
-     * Reset the headers.  Clear the write header and read in the next
-     * input header, if available.
+     * Reset the write header and read in the next input header, if available.
      */
-    RLECleanHeader (&destFilePtr->rleHeader, TRUE); /* Clean comments */
-    InitRLEHeader (destFilePtr);
-    
-    if (!ReadRLEHeader (tSippGlobPtr, srcFilePtr, srcHandle))
+    RLEResetHeader (&destFilePtr->rleHeader);
+    if (!ReadRLEHeader (tSippGlobPtr, srcFilePtr))
         return FALSE;
     
     return TRUE;
@@ -1369,7 +1446,7 @@ RLECopyImage (tSippGlobPtr, srcHandle, destHandle, destClassPtr, clear)
     if (srcFilePtr == NULL)
         return FALSE;
 
-    if (!srcFilePtr->readable) {
+    if ((srcFilePtr->perms & TCL_FILE_READABLE) == 0) {
         TSippNotReadable (tSippGlobPtr->interp, srcHandle);
         return FALSE;
     }
@@ -1394,9 +1471,9 @@ RLECopyImage (tSippGlobPtr, srcHandle, destHandle, destClassPtr, clear)
     outputParms.imgData.ySize = (srcFilePtr->rleHeader.ymax -
                                  srcFilePtr->rleHeader.ymin) + 1;
 
-    outputParms.imgData.mode         = MODE_UNKNOWN24;
+    outputParms.imgData.mode = MODE_UNKNOWN24;
     outputParms.imgData.overSampling = 1;
-    outputParms.imgData.field        = BOTH;
+    outputParms.imgData.field = BOTH;
     
     outputParms.imgData.backgroundColor [TSIPP_RED] =
         srcFilePtr->rleHeader.bg_color [RLE_RED];
@@ -1415,10 +1492,10 @@ RLECopyImage (tSippGlobPtr, srcHandle, destHandle, destClassPtr, clear)
     /*
      * Initialize other output parms fields.
      */
-    outputParms.argv     = NULL;   
-    outputParms.update   = 0;
-    outputParms.maxDepth = 0;
-    outputParms.clear    = clear;
+    outputParms.tSippGlobPtr = tSippGlobPtr;
+    outputParms.argv = NULL;   
+    outputParms.update = 0;
+    outputParms.clear= clear;
 
     outputParms.bitMapOutput = (outputParms.imgData.mode == LINE) &&
                                (destClassPtr->bitMapOptimal);
@@ -1455,7 +1532,7 @@ RLECopyImage (tSippGlobPtr, srcHandle, destHandle, destClassPtr, clear)
     /*
      * Read in the next input header, if available.
      */
-    status2 = ReadRLEHeader (tSippGlobPtr, srcFilePtr, srcHandle);
+    status2 = ReadRLEHeader (tSippGlobPtr, srcFilePtr);
 
     return status1 && status2;
 
@@ -1532,13 +1609,13 @@ RLECleanUp (tSippGlobPtr)
 
 }
 
-#else /* TSIPP_NO_RLE */
+#else /* TSIPP_HAVE_RLE */
 
 /*=============================================================================
  * SippRLEIsNotHere --
  *   Implements the a command that returns a error indicates that the Utah
  * Raster Toolkit is not available.  If this module is compiled with
- * TSIPP_NO_RLE defined, all RLE commands are bound to this command.
+ * TSIPP_HAVE_RLE is not defined, all RLE commands are bound to this command.
  *
  * Note:
  *   This procedure has standard Tcl command calling sematics.  ClientData
@@ -1589,4 +1666,4 @@ TSippRLEInit (tSippGlobPtr)
 
 }
 
-#endif /* TSIPP_NO_RLE */
+#endif /* TSIPP_HAVE_RLE */
