@@ -12,7 +12,7 @@
  * no representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tSippRender.c,v 3.0 1993/01/20 06:44:00 markd Rel $
+ * $Id: tSippRender.c,v 5.0 1994/09/05 01:23:37 markd Rel $
  *-----------------------------------------------------------------------------
  * Scenes are rendered to one or more image storage object.  Each storage
  * class is defined by attributes stored in the tSippGlob.  An instance of
@@ -95,12 +95,13 @@
 /*
  * Output image storage object information for each of the objects being
  * written.  Some information is cached from the class entry for speed.
+ * Handle is stored as part of this memory allocation unit.
  */
 typedef struct outputInfo_t {
     void                 *clientData;     /* Client data for output instance */
     void                (*outputLine) (); /* Call for each scan line.        */
     tSippStorageClass_pt  classPtr;       /* Class entry.                    */
-    char                  handle [16];    /* Handle associated with object.  */
+    char                 *handle;         /* Handle associated with object.  */
     bool                  initialized;    /* Startup code has been run.      */
     struct outputInfo_t  *nextPtr;        /* Next output object.             */
 } outputInfo_t, *outputInfo_pt;
@@ -380,8 +381,8 @@ AddStandardComment (outputParmsPtr, commentsPtr)
 {
     char  *infoArgv [5], *infoList, *bufPtr;
     char   overSamplingStr [16];
-    char   backgroundColorStr [3 * TSIPP_DOUBLE_STR_SIZE];
-    char   lineColorStr [3 * TSIPP_DOUBLE_STR_SIZE];
+    char   backgroundColorStr [3 * TCL_DOUBLE_SPACE];
+    char   lineColorStr [3 * TCL_DOUBLE_SPACE];
 
     TSippPutCom (commentsPtr, "TSIPP_VERSION", TSIPP_VERSION);
 
@@ -421,39 +422,51 @@ AddStandardComment (outputParmsPtr, commentsPtr)
         break;
     }
 
-    bufPtr = TSippFormatDouble (
+    /*
+     * Save the background color.
+     */
+    bufPtr = backgroundColorStr;
+    Tcl_PrintDouble (outputParmsPtr->tSippGlobPtr->interp,
         ((double) outputParmsPtr->imgData.backgroundColor [TSIPP_RED]) /
-                  255.0,
-        backgroundColorStr);
+                     255.0,
+         bufPtr);
+    bufPtr = bufPtr + strlen (bufPtr);
     *bufPtr++ = ' ';
 
-    bufPtr = TSippFormatDouble (
+    Tcl_PrintDouble (outputParmsPtr->tSippGlobPtr->interp,
         ((double) outputParmsPtr->imgData.backgroundColor [TSIPP_GREEN]) /
-                  255.0,
+                     255.0,
         bufPtr);
+    bufPtr = bufPtr + strlen (bufPtr);
     *bufPtr++ = ' ';
 
-    TSippFormatDouble (
+    Tcl_PrintDouble (outputParmsPtr->tSippGlobPtr->interp,
         ((double) outputParmsPtr->imgData.backgroundColor [TSIPP_BLUE]) /
                   255.0,
         bufPtr);
     infoArgv [3] = backgroundColorStr;
 
-    bufPtr = TSippFormatDouble (
+    /*
+     * Save the line color.
+     */
+    bufPtr = lineColorStr;
+    Tcl_PrintDouble (outputParmsPtr->tSippGlobPtr->interp,
         ((double) outputParmsPtr->imgData.lineColor [TSIPP_RED]) /
-                  255.0,
-       lineColorStr);
+                     255.0,
+       bufPtr);
+    bufPtr = bufPtr + strlen (bufPtr);
     *bufPtr++ = ' ';
 
-    bufPtr = TSippFormatDouble (
+    Tcl_PrintDouble (outputParmsPtr->tSippGlobPtr->interp,
         ((double) outputParmsPtr->imgData.lineColor [TSIPP_GREEN]) /
-                  255.0,
+                     255.0,
         bufPtr);
+    bufPtr = bufPtr + strlen (bufPtr);
     *bufPtr++ = ' ';
 
-    TSippFormatDouble (
+    Tcl_PrintDouble (outputParmsPtr->tSippGlobPtr->interp,
         ((double) outputParmsPtr->imgData.lineColor [TSIPP_BLUE]) /
-                  255.0,
+                     255.0,
         bufPtr);
     infoArgv [4] = lineColorStr;
 
@@ -563,12 +576,19 @@ SetupOutputEntry (tSippGlobPtr, outputParmsPtr, handle, outputListPtrPtr)
      */
     outputParmsPtr->scanDirection = storageClassPtr->scanDirection;
     
-    outputInfoPtr = (outputInfo_pt) smalloc (sizeof (outputInfo_t));
+    /*
+     * Set up output info, save the handle as part of it.  Add it into the
+     * list.
+     */
+    outputInfoPtr = (outputInfo_pt) smalloc (sizeof (outputInfo_t) +
+                                             strlen (handle) + 1);
     
     outputInfoPtr->clientData  = NULL;
     outputInfoPtr->outputLine  = storageClassPtr->outputLine;
     outputInfoPtr->classPtr    = storageClassPtr;
     outputInfoPtr->initialized = FALSE;
+    outputInfoPtr->handle      = PTR_ADD (outputInfoPtr,
+                                          sizeof (outputInfo_t));
     strcpy (outputInfoPtr->handle, handle);
 
     outputInfoPtr->nextPtr = *outputListPtrPtr;
@@ -974,12 +994,12 @@ ParseRenderParms (tSippGlobPtr, argc, argv, outputParmsPtr, handleListPtr)
     outputParmsPtr->imgData.lineColor [TSIPP_BLUE] =
         tSippGlobPtr->lineColor.blu * 255;
 
-    outputParmsPtr->argv           = argv;
-    outputParmsPtr->update         = -1;
-    outputParmsPtr->maxDepth       = 1024;
-    outputParmsPtr->clear          = TRUE;
-    outputParmsPtr->bitMapOutput   = FALSE;
-    outputParmsPtr->scanDirection  = TSIPP_SCAN_BOTH;
+    outputParmsPtr->tSippGlobPtr = tSippGlobPtr;
+    outputParmsPtr->argv = argv;
+    outputParmsPtr->update = -1;
+    outputParmsPtr->clear = TRUE;
+    outputParmsPtr->bitMapOutput = FALSE;
+    outputParmsPtr->scanDirection = TSIPP_SCAN_BOTH;
 
     if (!ParseRenderFlags (tSippGlobPtr, argv, outputParmsPtr, &nextArg))
         return FALSE;
@@ -1087,7 +1107,11 @@ SippRender (clientData, interp, argc, argv)
     if (outputParms.scanDirection == TSIPP_SCAN_BOTH) {
         outputParms.scanDirection = TSIPP_SCAN_TOP_DOWN;
     }
-    sipp_render_direction (outputParms.scanDirection == TSIPP_SCAN_BOTTOM_UP);
+
+    if (outputParms.scanDirection == TSIPP_SCAN_BOTTOM_UP)
+        sipp_render_direction (BOTTOM_TO_TOP);
+    else
+        sipp_render_direction (TOP_TO_BOTTOM);
 
     if (tSippGlobPtr->updateProc != NULL && outputParms.update > 0)
         sipp_set_update_callback (tSippGlobPtr->updateProc,
