@@ -11,7 +11,7 @@
  * no representations about the suitability of this software for any purpose.
  * It is provided "as is" without express or implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tSippUtil.c,v 3.0 1993/01/20 06:43:49 markd Rel $
+ * $Id: tSippUtil.c,v 5.0 1994/09/05 01:23:31 markd Rel $
  *============================================================================
  */
 
@@ -32,69 +32,6 @@ FindComment _ANSI_ARGS_((char  **comments,
                          char   *name,
                          int    *numComPtr));
 
-/*=============================================================================
- * TSippFormatDouble --
- *
- * Format a double to the maximum precision supported on this machine.  If the
- * number formats to an even integer, a ".0" is append to assure that the
- * value continues to represent a floating point number.
- *
- * Parmeters:
- *   o number (I) - The double to format.
- *   o buffer (O) - The buffer to store the resulting string in.  There should
- *     be at least TSIPP_DOUBLE_STR_SIZE bytes in the buffer.
- * Returns:
- *   A pointer to the terminating '\0' in the string.
- *-----------------------------------------------------------------------------
- */
-char *
-TSippFormatDouble (number, buffer)
-    double  number;
-    char   *buffer;
-{
-    static int precision = 0;
-    register char *scanPtr;
-
-    /*
-     * On the first call, determine the number of decimal digits that represent
-     * the precision of a double.
-     */
-    if (precision == 0) {
-	sprintf (buffer, "%.0f", pow (2.0, (double) DSIGNIF));
-	precision = strlen (buffer);
-    }
-
-    sprintf (buffer, "%.*g", precision, number);
-
-    /*
-     * Scan the number for "." or "e" to assure that the number has not been
-     * converted to an integer.	 Also check for NaN on infinite
-     */
-
-    scanPtr = buffer;
-    if (scanPtr [0] == '-')
-	scanPtr++;
-    for (; *scanPtr != '\0' ; scanPtr++) {
-        if ((*scanPtr == '.') || (*scanPtr == 'e'))
-            break;
-    }
-
-    /*
-     * Make it a float or find the end.
-     */
-    if (*scanPtr == '\0') {
-        scanPtr [0] = '.';
-        scanPtr [1] = '0';
-        scanPtr [2] = '\0';
-        scanPtr += 2;
-    } else {
-        for (; *scanPtr != '\0' ; scanPtr++)
-            continue;
-    }
-    return scanPtr;
-
-}
-     
 /*=============================================================================
  * TSippConvertFraction --
  *   Convert a string that should be in the range 0 to 1 to a double and do
@@ -113,7 +50,6 @@ TSippConvertFraction (tSippGlobPtr, strBuf, valuePtr)
     char         *strBuf;
     double       *valuePtr;
 {
-
     if (Tcl_GetDouble (tSippGlobPtr->interp, strBuf, valuePtr) != TCL_OK)
         return FALSE;
     if ((*valuePtr < 0.0) || (*valuePtr > 1.0)) {
@@ -123,7 +59,6 @@ TSippConvertFraction (tSippGlobPtr, strBuf, valuePtr)
         return FALSE;
     }
     return TRUE;
-
 }
 
 /*=============================================================================
@@ -143,7 +78,6 @@ TSippConvertUnsignedDbl (tSippGlobPtr, strBuf, valuePtr)
     char         *strBuf;
     double       *valuePtr;
 {
-
     if (Tcl_GetDouble (tSippGlobPtr->interp, strBuf, valuePtr) != TCL_OK)
         return FALSE;
     if (*valuePtr < 0.0) {
@@ -153,7 +87,6 @@ TSippConvertUnsignedDbl (tSippGlobPtr, strBuf, valuePtr)
         return FALSE;
     }
     return TRUE;
-
 }
 
 /*=============================================================================
@@ -173,7 +106,6 @@ TSippConvertPosUnsigned (tSippGlobPtr, strBuf, valuePtr)
     char         *strBuf;
     unsigned     *valuePtr;
 {
-
     if (Tcl_GetUnsigned (tSippGlobPtr->interp, strBuf, valuePtr) != TCL_OK)
         return FALSE;
     if (*valuePtr == 0) {
@@ -182,7 +114,6 @@ TSippConvertPosUnsigned (tSippGlobPtr, strBuf, valuePtr)
         return FALSE;
     }
     return TRUE;
-
 }
 
 /*=============================================================================
@@ -213,8 +144,8 @@ TSippConvertColor (tSippGlobPtr, listStr, colorPtr)
 
     if (listArgc != 3) {
         Tcl_AppendResult (tSippGlobPtr->interp,
-                          "color must be a list of three elements",
-                          (char *) NULL);
+                          "color must be a list of three elements, got \"",
+                          listStr, "\"", (char *) NULL);
         goto errorCleanup;
     }
     if (!TSippConvertFraction (tSippGlobPtr, listArgv [0], &colorPtr->red))
@@ -230,7 +161,6 @@ TSippConvertColor (tSippGlobPtr, listStr, colorPtr)
 errorCleanup:
     sfree (listArgv);
     return FALSE;
-
 }
 
 /*=============================================================================
@@ -262,8 +192,8 @@ TSippConvertOpacity (tSippGlobPtr, listStr, opacityPtr)
 
     if (!((listArgc == 1) || (listArgc == 3))) {
         Tcl_AppendResult (tSippGlobPtr->interp, "opacity must be a single ",
-                          "number or a list of three elements",
-                          (char *) NULL);
+                          "number or a list of three elements, got \"",
+                          listStr, "\"", (char *) NULL);
         goto errorCleanup;
     }
     if (listArgc == 1) {
@@ -288,7 +218,6 @@ TSippConvertOpacity (tSippGlobPtr, listStr, opacityPtr)
 errorCleanup:
     sfree (listArgv);
     return FALSE;
-
 }
 
 /*=============================================================================
@@ -323,7 +252,6 @@ TSippConvertAngleRad (tSippGlobPtr, strBuf, valuePtr)
     if (strBuf [0] == 'D')
        *valuePtr = (*valuePtr * M_PI) / 180.0;
     return TRUE;
-
 }
 
 /*=============================================================================
@@ -358,7 +286,6 @@ TSippConvertAngleDeg (tSippGlobPtr, strBuf, valuePtr)
     if (strBuf [0] == 'R')
        *valuePtr = (*valuePtr * 180.0) / M_PI;
     return TRUE;
-
 }
 
 /*=============================================================================
@@ -391,8 +318,8 @@ TSippConvert2DPoint (tSippGlobPtr, listStr, xCoordPtr, yCoordPtr)
 
     if (listArgc != 2) {
         Tcl_AppendResult (tSippGlobPtr->interp, 
-                          "a 2D point must be a list of two elements",
-                          (char *) NULL);
+                          "a 2D point must be a list of two elements, got \"",
+                          listStr, "\"", (char *) NULL);
         goto errorCleanup;
     }
    
@@ -409,7 +336,6 @@ TSippConvert2DPoint (tSippGlobPtr, listStr, xCoordPtr, yCoordPtr)
 errorCleanup:
     sfree (listArgv);
     return FALSE;
-
 }
 
 /*=============================================================================
@@ -440,7 +366,8 @@ TSippConvertVertex (tSippGlobPtr, listStr, vertexPtr)
 
     if (vertexArgc != 3) {
         Tcl_AppendResult (tSippGlobPtr->interp, "vertex or vector must be",
-                         " a list of three elements", (char *) NULL);
+                         " a list of three elements, got \"", listStr,
+                          "\"", (char *) NULL);
         goto errorCleanup;
     }
 
@@ -459,79 +386,6 @@ TSippConvertVertex (tSippGlobPtr, listStr, vertexPtr)
 errorCleanup:
     sfree (vertexArgv);
     return FALSE;
-
-}
-
-/*=============================================================================
- * TSippConvertVertexTex --
- *   Convert a list contain a list of two triples of numbers repersenting a
- * vertex X, Y and Z and texture U, V and W into vertex and texture
- * coordinates.
- *
- * Parameters:
- *   o tSippGlobPtr (I) - A pointer to the Tcl SIPP global structure.
- *   o listStr (I) - A Tcl list containing the vertex values to convert.
- *   o vertexPtr (O) - The converted vertex is returned here.
- *   o texturePtr (O) - The converted texture coordinates are returned here.
- * Returns:
- *   TRUE if the list and numbers are valid, FALSE if there is an error.
- *-----------------------------------------------------------------------------
- */
-bool
-TSippConvertVertexTex (tSippGlobPtr, listStr, vertexPtr, texturePtr)
-    tSippGlob_pt  tSippGlobPtr;
-    char         *listStr;
-    Vector       *vertexPtr;
-    Vector       *texturePtr;
-{
-    int      pairArgc,  coordArgc, idx;
-    char   **pairArgv, **coordArgv;
-    Vector  *currentPtr;
-
-    if (Tcl_SplitList (tSippGlobPtr->interp, listStr, &pairArgc,
-                       &pairArgv) != TCL_OK)
-        return FALSE;
-
-    if (pairArgc != 2) {
-        Tcl_AppendResult (tSippGlobPtr->interp, "vertex-texture list must be",
-                          " a list of two coordinates", (char *) NULL);
-        goto errorCleanup;
-    }
-
-    currentPtr = vertexPtr;
-    for (idx = 0; idx < 2; idx++) {
-        if (Tcl_SplitList (tSippGlobPtr->interp, pairArgv [idx], &coordArgc,
-                           &coordArgv) != TCL_OK)
-            goto errorCleanup;
-
-        if (coordArgc != 3) {
-            Tcl_AppendResult (tSippGlobPtr->interp, "vertex or point must be",
-                             " a list of three elements", (char *) NULL);
-            goto errorCleanup2;
-        }
-
-        if (Tcl_GetDouble (tSippGlobPtr->interp, coordArgv [0],
-                           &currentPtr->x) != TCL_OK)
-           goto errorCleanup2;
-        if (Tcl_GetDouble (tSippGlobPtr->interp, coordArgv [1],
-                           &currentPtr->y) != TCL_OK)
-           goto errorCleanup2;
-        if (Tcl_GetDouble (tSippGlobPtr->interp, coordArgv [2],
-                           &currentPtr->z) != TCL_OK)
-           goto errorCleanup2;
-
-        sfree (coordArgv);
-        currentPtr = texturePtr;
-    }
-    sfree (pairArgv);
-    return TRUE;
-
-errorCleanup2:
-    sfree (coordArgv);
-errorCleanup:
-    sfree (pairArgv);
-    return FALSE;
-
 }
 
 /*=============================================================================
@@ -565,8 +419,8 @@ TSippConvertMatrix (tSippGlobPtr, listStr, matrixPtr)
                        &rowArgv) != TCL_OK)
         return FALSE;
     if (numRows != 4) {
-        Tcl_AppendResult (tSippGlobPtr->interp, "matrix must have 4 rows",
-                          (char *) NULL);
+        Tcl_AppendResult (tSippGlobPtr->interp, "matrix must have 4 rows, ",
+                          "got \"", listStr, "\"", (char *) NULL);
         goto rowErrorExit;
     }
     for (row = 0; row < 4; row++) {
@@ -575,7 +429,8 @@ TSippConvertMatrix (tSippGlobPtr, listStr, matrixPtr)
             goto rowErrorExit;
         if (numColumns != 4) {
             Tcl_AppendResult (tSippGlobPtr->interp, 
-                              "matrix must have 4 columns", (char *) NULL);
+                              "matrix must have 4 columns, got \"",
+                              rowArgv [row], "\"", (char *) NULL);
             goto colErrorExit;
         }
         for (column = 0; column < 3; column++) {
@@ -609,7 +464,6 @@ colErrorExit:
 rowErrorExit:
     sfree (rowArgv);
     return FALSE;
-
 }
 
 /*=============================================================================
@@ -617,18 +471,20 @@ rowErrorExit:
  *   Format a 4x4 matrix as a list of lists.
  *
  * Parameters:
+ *   o tSippGlobPtr (I) - A pointer to the Tcl SIPP global structure.
  *   o matrixPtr (I) - The matrix to format.
  * Returns:
  *   A dynamically alloced string contain the lists.
  *-----------------------------------------------------------------------------
  */
 char *
-TSippFormatMatrix (matrixPtr)
+TSippFormatMatrix (tSippGlobPtr, matrixPtr)
+   tSippGlob_pt  tSippGlobPtr;
    Transf_mat   *matrixPtr;
 {
     int    row; 
     char  *rowArgv [4], *bufPtr;
-    char   rowBuffers [4][4 * (TSIPP_DOUBLE_STR_SIZE + 1)];
+    char   rowBuffers [4][4 * TCL_DOUBLE_SPACE];
 
     rowArgv [0] = rowBuffers [0];
     rowArgv [1] = rowBuffers [1];
@@ -640,16 +496,30 @@ TSippFormatMatrix (matrixPtr)
      */
     for (row = 0; row < 4; row++) {
         bufPtr = rowArgv [row];
-        bufPtr = TSippFormatDouble (matrixPtr->mat [row][0], bufPtr);
-        *bufPtr = ' ';
-        bufPtr = TSippFormatDouble (matrixPtr->mat [row][1], bufPtr + 1);
-        *bufPtr = ' ';
-        bufPtr = TSippFormatDouble (matrixPtr->mat [row][2], bufPtr + 1);
-        *bufPtr = ' ';
-        TSippFormatDouble (matrixCol3 [row], bufPtr + 1);
+
+        Tcl_PrintDouble (tSippGlobPtr->interp,
+                         matrixPtr->mat [row][0],
+                         bufPtr);
+        bufPtr = bufPtr + strlen (bufPtr);
+        *bufPtr++ = ' ';
+
+        Tcl_PrintDouble (tSippGlobPtr->interp,
+                         matrixPtr->mat [row][1],
+                         bufPtr);
+        bufPtr = bufPtr + strlen (bufPtr);
+        *bufPtr++ = ' ';
+
+        Tcl_PrintDouble (tSippGlobPtr->interp,
+                         matrixPtr->mat [row][2],
+                         bufPtr);
+        bufPtr = bufPtr + strlen (bufPtr);
+        *bufPtr++ = ' ';
+
+        Tcl_PrintDouble (tSippGlobPtr->interp,
+                         matrixCol3 [row],
+                         bufPtr);
     }
     return Tcl_Merge (4, rowArgv);
-
 }
 
 /*=============================================================================
@@ -756,7 +626,6 @@ errorExit:
         TSippHandleListFree (handleEntryListPtr);
     sfree (handleArgv);
     return FALSE;
-
 }
 
 /*=============================================================================
@@ -773,7 +642,6 @@ TSippHandleListFree (handleListPtr)
 {
     if (handleListPtr->ptr != handleListPtr->staticArray)
         sfree (handleListPtr->ptr);
-
 }
 
 /*=============================================================================
@@ -798,7 +666,6 @@ TSippInitCmds (tSippGlobPtr, cmdTablePtr)
                           (ClientData) tSippGlobPtr, (void (*)())NULL);
         cmdTablePtr++;
     }
-
 }
 
 /*=============================================================================
@@ -830,7 +697,6 @@ TSippAddStorageClass (tSippGlobPtr, storageClassPtr)
 
     tSippGlobPtr->storageClassPtrs [idx]     = storageClassPtr;
     tSippGlobPtr->storageClassPtrs [idx + 1] = NULL;
-
 }
 
 /*=============================================================================
@@ -892,7 +758,6 @@ TSippParseTextureMapping (tSippGlobPtr, textureStr, texturePtr, invalidList)
 
     *texturePtr = texture;
     return TRUE;
-
 }
 
 /*=============================================================================
@@ -913,7 +778,6 @@ TSippNotWhileRendering (interp)
 {
     interp->result = "SIPP state may not be modified while rendering";
     return TCL_ERROR;
-
 }
 
 /*=============================================================================
@@ -936,7 +800,6 @@ TSippAtEOF (interp, handle)
     Tcl_AppendResult (interp, handle, " is at the end of file",
                       (char *) NULL);
     return TCL_ERROR;
-
 }
 
 /*=============================================================================
@@ -959,7 +822,6 @@ TSippNotReadable (interp, handle)
     Tcl_AppendResult (interp, handle, " is not open for read access",
                       (char *) NULL);
     return TCL_ERROR;
-
 }
 
 /*=============================================================================
@@ -982,7 +844,6 @@ TSippNotWritable (interp, handle)
     Tcl_AppendResult (interp, handle, " is not open for write access",
                       (char *) NULL);
     return TCL_ERROR;
-
 }
 
 /*=============================================================================
@@ -1027,7 +888,7 @@ FindComment (comments, name, numComPtr)
     }
 
     return (comments [idx] == NULL) ? -1 : idx;
-    
+   
 }
 
 /*=============================================================================
@@ -1087,7 +948,6 @@ TSippPutCom (commentsPtr, name, value)
     }
 
     *commentsPtr = comments;
-
 }
 
 /*=============================================================================
@@ -1118,7 +978,6 @@ TSippGetCom (comments, name)
         len++;
 
     return &(comments [idx] [len]);
-
 }
 
 /*=============================================================================
@@ -1180,7 +1039,6 @@ TSippGetAllCom (comments)
         sfree (elements [idx]);
 
     return keyedList;
-
 }
 
 /*=============================================================================
@@ -1239,7 +1097,6 @@ TSippFreeCom (commentsPtr)
     sfree (comments);
 
     *commentsPtr = NULL;
-    
 }
 
 /*=============================================================================
@@ -1272,7 +1129,6 @@ TSippCopyCom (comments)
     newComments [idx] = NULL;
 
     return newComments;
-    
 }
 
 /*=============================================================================
@@ -1390,6 +1246,5 @@ TSippAddHist (commentsPtr, argv)
     strcat (newHistory, "\t");
 
     TSippPutCom (commentsPtr, "HISTORY", newHistory);
-
 }
 
